#!/usr/bin/env python3
#
# CMPUT 229 Public Materials License
# Version 1.0
#
# Copyright 2017 University of Alberta
# Copyright 2017 Kristen Newbury
# Copyright 2024 Rajan Maghera
#
# This software is distributed to students in the course
# CMPUT 229 - Computer Organization and Architecture I at the University of
# Alberta, Canada.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
'''
Author: Kristen Newbury
Date: August 4 2017


Takes a .txt file as input and produces a .bin file
containing the binary word representation of each int(separated by newline) in the filename

sample usage:

    python3 convert.py sample.txt [output file]

Author: Rajan Maghera
Date: January 27 2024

Modifications to add better validation support and semantics.

'''

import sys, re

def read_numbers_from_file(file_name):
    '''
    Reads a list of numbers from a file, one number per line
    '''

    numbers = []
    with open(file_name) as f:
        lines = f.readlines()

        # Read numbers from file
        try:
            numbers = [ int(f.strip()) for f in lines ]
        except ValueError:
            print("Error: .txt file should only contain integers")
            sys.exit(1)

    if len(numbers) <= 0:
        print("Error: unknown error occured while reading file")
        sys.exit(1)

    return numbers

def write_numbers_to_binary_file(file_name, numbers):
    '''
    Writes a list of numbers to a binary file
    '''

    with open(file_name, "wb") as f:
        try:
            for n in numbers:
                f.write((n).to_bytes(4, byteorder='little', signed='True'))
        except:
            print("Error: unknown error occured while writing to file")
            sys.exit(1)

def main():
    '''
    Main function
    '''

    if len(sys.argv) < 2:
        print("No text file names supplied")
        sys.exit(1)

    # Get the name of the input file
    txt_file_name = sys.argv[1]

    # Read numbers from file
    numbers = read_numbers_from_file(txt_file_name)

    # Check that the numbers are all greater than -4 (numbers can only ever be positive + -1, -2, -3)
    if any([ n <= -4 for n in numbers ]):
        print("Error: .txt file should only contain integers greater than -4")
        sys.exit(1)

    # Check that the list of number is more than 2
    if len(numbers) < 2:
        print("Error: .txt file should contain at least 4 integers (1 operand and sentinel value")
        sys.exit(1)

    # Check that the last number is the sentinel -3
    if numbers[-1] != -3:
        print("Error: .txt file should end with -3")
        sys.exit(1)

    # Check that the number of negatives matches the number of positives
    # Number of negatives = number of positives (operands) - 1 + 1 for sentinel
    if len([ n for n in numbers if n < 0 ]) != len([ n for n in numbers if n >= 0 ]):
        print("Error: .txt file should contain the same number of negative and positive integers")
        sys.exit(1)


    # Substitute extension on input, go from .txt to .bin file for output
    # If the user provides a second file name, use that as the output
    bin_file_name = re.sub('.txt$', '.bin', txt_file_name) if len(sys.argv) < 3 else sys.argv[2]

    # Write numbers to binary file
    write_numbers_to_binary_file(bin_file_name, numbers)

if __name__ == "__main__":
    main()

